CHAR_OWNED_VERSION = GetAddOnMetadata("CharacterOwnedCount", "Version");
CHAR_OWNED_VERSIONID = 10700;
CHAR_OWNED_NAME = "";
CHAR_OWNED_REALM = "";
CHAR_OWNED_PROFILE = "";
CHAR_OWNED = {};
CHAR_OWNED_UNLOCKED = false;

local CHAR_OWNED_TIMER = 0;
local CHAR_OWNED_STATE = 0;
local CHAR_OWNED_WAIT = false;
local CHAR_OWNED_WAIT_TIMER = 0;
local CHAR_OWNED_COOLDOWN = 0;
local CHAR_OWNED_PLAYERDEAD = false;
local CHAR_OWNED_INSTANCEMODE = false;
local CHAR_OWNED_INSTANCE_COUNT = 0;
local LAST_INSTANCE_TYPE = "";
local CHECK_INSTANCE_TIMER = 0;

local L = LibStub("AceLocale-3.0"):GetLocale("CharacterOwnedCount", true);

StaticPopupDialogs["CHAR_OWNED_CONFIRM_RESET"] = {
    text = L["CONFIRM_RESET_COUNT"],
    button1 = YES,
    button2 = NO,
    OnAccept = function (self) CharOwned_StartReset(); end,
    OnCancel = function (self) end,
    timeout = 0,
    hideOnEscape = 1,
};

StaticPopupDialogs["CHAR_OWNED_INSTANCE_PROMPT"] = {
    text = L["ENABLE_INSTANCE_COUNTS"],
    button1 = YES,
    button2 = NO,
    OnAccept = function (self) CharOwned_SetGroupSeparation(true); end,
    OnCancel = function (self) CharOwned_SetGroupSeparation(false); end,
    timeout = 0,
    hideOnEscape = 1,
};

--========================================
-- Initial load routine
--========================================
function CharOwned_OnLoad(self)

    -- register events
    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("PLAYER_DEAD");
	self:RegisterEvent("PLAYER_ALIVE");
	self:RegisterEvent("PLAYER_UNGHOST");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");

    CharOwnedCount:RegisterForDrag("LeftButton");

    -- register slash command
    SLASH_CHAROWNED1 = "/charowned";
    SlashCmdList["CHAROWNED"] = CharOwned_CommandHandler;

    -- the death event fires when logging in on a character that's already dead,
    -- so activate a 0.5 second 'cooldown' timer so it can't trigger the
    -- counter upon logging in
    CharOwned_Cooldown(0.5);

end

--========================================
-- Event triggers
--========================================
function CharOwned_OnEvent(self, event, ...)

    local arg1 = ...;

    if (event == "ADDON_LOADED") then
        if (arg1 == "CharacterOwnedCount") then
            CharOwned_Setup();
        end
    elseif (event == "PLAYER_DEAD") then
        if ((CHAR_OWNED_WAIT ~= true) and (CHAR_OWNED_PLAYERDEAD ~= true)) then
            CharOwned_OnOwned();
			CHAR_OWNED_PLAYERDEAD = true;
        end
	elseif ((event == "PLAYER_ALIVE") or (event == "PLAYER_UNGHOST")) then
		CHAR_OWNED_PLAYERDEAD = false;
    elseif (event == "PLAYER_ENTERING_WORLD") then
        --CharOwned_UpdateInstanceMode();
        CHECK_INSTANCE_TIMER = 0.5;
    end

end

--========================================
-- Main setup routine
--========================================
function CharOwned_Setup()

    -- store the current character name and realm name
	CHAR_OWNED_NAME = UnitName("player");
    CHAR_OWNED_REALM = GetRealmName();
	CHAR_OWNED_PROFILE = CHAR_OWNED_REALM .. "." .. CHAR_OWNED_NAME;

    local storedVersion, isUpdate = CharOwned_CheckSetting("version", CHAR_OWNED_VERSIONID);
	CharOwned_CheckSetting("duration", 4);
	CharOwned_CheckSetting("enablesound", true);
    CharOwned_CheckSetting("show_load_message", false);
    CharOwned_CheckSetting("separate_instance_counts", false);
    CharOwned_CheckSetting("prompt_instance_mode", true);

    -- if upgrading from old versions (1.6.3 or lower), convert old profiles
    if (isUpdate) then
        local k, v;
        for k, v in pairs(CHAR_OWNED) do
            if (k ~= "config") then
                CHAR_OWNED[k] = { ["owned_count"] = v };
            end
        end
    end

    CharOwned_CheckProfileSetting("owned_count", 0);

    -- set text strings
    CharOwned_InitText();

    -- another login cooldown, just to be sure
    CharOwned_Cooldown(0.5);

    if (CHAR_OWNED['config']['show_load_message']) then
        CharOwned_Message(string.format(L["LOADED_MESSAGE"], CHAR_OWNED_VERSION));
    end

    CharOwned_LockFrame(true, true);

end

--========================================
-- Check configuration setting, and
-- initialize with default value if not
-- present
--========================================
function CharOwned_CheckSetting(field, defaultValue)

    local isNew = true;

    -- make sure the config array is present
    if (CHAR_OWNED['config'] == nil) then
	    CHAR_OWNED['config'] = {};
	end

	if (CHAR_OWNED['config'][field] == nil) then
	    CHAR_OWNED['config'][field] = defaultValue;
    else
        isNew = false;
	end

    return CHAR_OWNED['config'][field], isNew;

end

--========================================
-- Check profile-specific value, and
-- initialize with default value if not
-- present
--========================================
function CharOwned_CheckProfileSetting(field, defaultValue)

    -- make sure the config array is present
    if (CHAR_OWNED[CHAR_OWNED_PROFILE] == nil) then
	    CHAR_OWNED[CHAR_OWNED_PROFILE] = {};
	end

	if (CHAR_OWNED[CHAR_OWNED_PROFILE][field] == nil) then
	    CHAR_OWNED[CHAR_OWNED_PROFILE][field] = defaultValue;
	end

end

--========================================
-- Set default text
--========================================
function CharOwned_InitText()
    CharOwnedCountText1:SetText(CHAR_OWNED_NAME);
    CharOwnedCountText2:SetText(L["OWNED_COUNT1"]);
    CharOwned_UpdateCount();
end

--========================================
-- Main update routine
--========================================
function CharOwned_OnUpdate(self, elapsed)

    if ((CHAR_OWNED_SHOWN == true) and (CHAR_OWNED_UNLOCKED ~= true)) then
        -- wait 0.5 sec before showing the counter on the screen
        if ((CHAR_OWNED_STATE == 1) or (CHAR_OWNED_STATE == 4)) then
            if (GetTime() >= (CHAR_OWNED_TIMER + 0.5)) then
                CharOwnedCount:Show();
                CHAR_OWNED_STATE = CHAR_OWNED_STATE + 1;
            end
        -- another 0.25 sec before incrementing the count and playing the sound
        elseif (CHAR_OWNED_STATE == 2) then
            if (GetTime() >= (CHAR_OWNED_TIMER + 0.75)) then
                CharOwned_Increase();
                CHAR_OWNED_STATE = 3;
            end
        -- and then hide the counter again after another 3 seconds
        elseif (CHAR_OWNED_STATE == 3) then
            if (GetTime() >= (CHAR_OWNED_TIMER + (0.75 + CHAR_OWNED['config']['duration']))) then
                CharOwnedCount:Hide();
                CHAR_OWNED_SHOWN = false;
                CHAR_OWNED_STATE = 0;
            end

        -- 0.25 sec before resetting the count and playing the sound
        elseif (CHAR_OWNED_STATE == 5) then
            if (GetTime() >= (CHAR_OWNED_TIMER + 0.75)) then
                CharOwned_DoReset();
                CHAR_OWNED_STATE = 3;
            end
        end
    end

    -- cooldown timer
    if (CHAR_OWNED_WAIT) then
        if (GetTime() >= (CHAR_OWNED_WAIT_TIMER + CHAR_OWNED_COOLDOWN)) then
            CharOwned_Cooldown(0);
        end
    end
    
    if (CHECK_INSTANCE_TIMER > 0) then
        CHECK_INSTANCE_TIMER = math.max(0, CHECK_INSTANCE_TIMER - elapsed);
        if (CHECK_INSTANCE_TIMER == 0) then CharOwned_UpdateInstanceMode(); end
    end

end

--========================================
-- Main counter trigger, activated when
-- killed
--========================================
function CharOwned_OnOwned()
    CHAR_OWNED_TIMER = GetTime();
    CHAR_OWNED_SHOWN = true;
    CHAR_OWNED_STATE = 1;
end

--========================================
-- Main reset trigger
--========================================
function CharOwned_StartReset()
    CHAR_OWNED_TIMER = GetTime();
    CHAR_OWNED_SHOWN = true;
    CHAR_OWNED_STATE = 4;
end

--========================================
-- Increases the count and plays the sound
--========================================
function CharOwned_Increase()
    CharOwned_PlaySound();
    if (CHAR_OWNED_INSTANCEMODE) then
        CHAR_OWNED_INSTANCE_COUNT = CHAR_OWNED_INSTANCE_COUNT + 1;
    else
    end
    CHAR_OWNED[CHAR_OWNED_PROFILE]['owned_count'] = CHAR_OWNED[CHAR_OWNED_PROFILE]['owned_count'] + 1;
    CharOwned_UpdateCount();
end

--========================================
-- Resets the count and plays the sound
--========================================
function CharOwned_DoReset()
    CharOwned_PlaySound();
    CHAR_OWNED[CHAR_OWNED_PROFILE]['owned_count'] = 0;
    CharOwned_UpdateCount();
end

--========================================
-- Plays the 'ding' sound if sound is on
--========================================
function CharOwned_PlaySound()
    if (CHAR_OWNED['config']['enablesound'] == true) then
		PlaySoundFile("Interface\\AddOns\\CharacterOwnedCount\\Sounds\\owned-ding.ogg");
	end
end

--========================================
-- Updates the text display on the counter
--========================================
function CharOwned_UpdateCount()

    if (CHAR_OWNED_UNLOCKED ~= true) then
        CharOwnedCountText3:SetText(string.format(L["OWNED_COUNT2"], CharOwned_GetDisplayedCount()));
    end

end

--========================================
-- Returns the number to be displayed for
-- the owned count, depending on if using
-- a separate instance-only count
--========================================
function CharOwned_GetDisplayedCount()
    if (CHAR_OWNED_INSTANCEMODE) then
        return CHAR_OWNED_INSTANCE_COUNT;
    else
        return CHAR_OWNED[CHAR_OWNED_PROFILE]['owned_count'];
    end
end

--========================================
-- Triggers a cooldown timer, during
-- which the counter can't be triggered
--========================================
function CharOwned_Cooldown(delay)
    if (delay > 0) then
        CHAR_OWNED_COOLDOWN = delay;
        CHAR_OWNED_WAIT = true;
        CHAR_OWNED_WAIT_TIMER = GetTime();
    else
        CHAR_OWNED_WAIT = false;
        CHAR_OWNED_WAIT_TIMER = 0;
        CHAR_OWNED_COOLDOWN = 0;
    end
end

--========================================
-- Locks and hides the frame
--========================================
function CharOwned_LockFrame(silent, force)

    if ((CHAR_OWNED_UNLOCKED) or (force)) then
        CharOwnedCount:EnableMouse(false);
        CHAR_OWNED_UNLOCKED = false;
        CharOwned_InitText();
        CharOwned_ResetState();
    end

end

--========================================
-- Unlocks and shows the frame
--========================================
function CharOwned_UnlockFrame(silent)

    if (CHAR_OWNED_UNLOCKED ~= true) then
        CharOwnedCount:EnableMouse(true);
        CharOwnedCount:Show();
        CharOwnedCountText1:SetText(L["FRAME_UNLOCKED1"]);
        CharOwnedCountText2:SetText(L["FRAME_UNLOCKED2"]);
        CharOwnedCountText3:SetText(L["FRAME_UNLOCKED3"]);
        CHAR_OWNED_UNLOCKED = true;
    end

end

--========================================
-- Completely resets the state of the
-- counter, for when locking and
-- initializing
--========================================
function CharOwned_ResetState()
    CharOwnedCount:Hide();
    CharOwned_Cooldown(0);
    CHAR_OWNED_STATE = 0;
    CHAR_OWNED_SHOWN = false;
end

--========================================
-- Slash command (/charowned) handler
--========================================
function CharOwned_CommandHandler(cmd)

    InterfaceOptionsFrame_OpenToCategory(CharOwnedCountConfigContainer);

end

--========================================
-- Reset to default position
--========================================
function CharOwned_ResetPosition(silent)
    CharOwnedCount:SetUserPlaced(0);
    CharOwnedCount:ClearAllPoints();
    CharOwnedCount:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", -128, 128);
    CharOwnedCount:SetWidth(256);
    CharOwnedCount:SetHeight(128);
	if (silent ~= true) then
        CharOwned_Message(L["POSITION_RESET_MESSAGE"]);
	end
end

--========================================
-- Outputs status message to chat window
--========================================
function CharOwned_Message(text)
    DEFAULT_CHAT_FRAME:AddMessage("|cfff89028<" .. L["ADDON_TITLE"] .. ">|cffffffff " .. text, 1, 1, 1);
end

--========================================
-- Shows the current count (without
-- incrementing)
--========================================
function CharOwned_ShowCount()
    CHAR_OWNED_TIMER = GetTime() + 0.25;
    CHAR_OWNED_SHOWN = true;
    CHAR_OWNED_STATE = 3;
    CharOwnedCount:Show();
end

--========================================
-- Sets whether separate owned counts
-- should be used in instance groups
--========================================
function CharOwned_SetGroupSeparation(value)
    CHAR_OWNED['config']['separate_instance_counts'] = value;
    CHAR_OWNED['config']['prompt_instance_mode'] = false;
    CharOwned_UpdateInstanceMode();
end

--========================================
-- Controls switching to/from 'instance
-- mode' depending on conditions
--========================================
function CharOwned_UpdateInstanceMode()
    local isLFD = true;
    local mode, subMode = GetLFGMode(LE_LFG_CATEGORY_LFD);
    if ((mode ~= "lfgparty") and (mode ~= "abandonedInDungeon")) then
        mode, subMode = GetLFGMode(LE_LFG_CATEGORY_RF);
        if ((mode ~= "lfgparty") and (mode ~= "abandonedInDungeon")) then
            mode, subMode = GetLFGMode(LE_LFG_CATEGORY_SCENARIO);
            if ((mode ~= "lfgparty") and (mode ~= "abandonedInDungeon")) then
                isLFD = false;
            end
        end
    end
    local inInstance, instanceType = IsInInstance();
    --local isLFD = ((mode == "lfgparty") or (mode == "abandonedInDungeon"));
    local i;
    local isBattleground = false;
    for i = 0, 3, 1 do
        isBattleground = (isBattleground or (select(1, GetBattlefieldStatus(i)) == "active"));
    end
    
    if (isLFD or isBattleground) then
        if (inInstance) then
            if (CHAR_OWNED['config']['prompt_instance_mode'] == true) then
                StaticPopup_Show("CHAR_OWNED_INSTANCE_PROMPT");
                return;
            else
                CharOwned_SetInstanceMode(true, instanceType);
            end
        end
    else
        CharOwned_SetInstanceMode(false);
    end
end

--========================================
-- Performs the instance mode switch
--========================================
function CharOwned_SetInstanceMode(value, instanceType)
    instanceType = (instanceType or "");
    if ((CHAR_OWNED_INSTANCEMODE == value) and (LAST_INSTANCE_TYPE == instanceType)) then return; end
    CHAR_OWNED_INSTANCEMODE = value;
    LAST_INSTANCE_TYPE = instanceType;
    --if (CHAR_OWNED_INSTANCEMODE) then
    --    CharOwned_Message("TEST: Instance mode ON");
    --else
    --    CharOwned_Message("TEST: Instance mode OFF");
    --end
    CHAR_OWNED_INSTANCE_COUNT = 0;
end
