local CHAR_OWNED_LASTMUFFIN = 0;

local L = LibStub("AceLocale-3.0"):GetLocale("CharacterOwnedCount", true);

local OLD_ENABLE_SOUND = false;
local OLD_LOAD_MESSAGE = false;

local CONFIG_SHOWN = false;

--========================================
-- Set up the config frame
--========================================
function CharOwnedConfig_OnLoad(self)
    self.name = L["ADDON_TITLE"];
    self.okay = function(self) CharOwnedConfig_Close(); end;
    self.cancel = function(self) CharOwnedConfig_Cancel(); end;
    self.refresh = function(self) CharOwnedConfig_Refresh(); end;
    self.default = function(self) CharOwnedConfig_SetDefaults(); end;
    InterfaceOptions_AddCategory(self);

    CharOwnedCountConfigTitle:SetText(string.format(L["VERSION_TEXT"], "|cffffffffv" .. CHAR_OWNED_VERSION));

    -- ********** General Options **********

    CharOwnedCountConfig_GeneralOptionsTitle:SetText(L["CONFIG_HEADING_GENERAL"]);

	CharOwnedCountConfig_GeneralOptions_EnableSoundText:SetText(L["OPTION_ENABLESOUND"]);
	CharOwnedCountConfig_GeneralOptions_EnableSound.tooltip = L["OPTION_TOOLTIP_ENABLESOUND"];

	CharOwnedCountConfig_GeneralOptions_LoadMessageText:SetText(L["OPTION_LOAD_MESSAGE"]);
	CharOwnedCountConfig_GeneralOptions_LoadMessage.tooltip = L["OPTION_TOOLTIP_LOAD_MESSAGE"];

    -- ********** Frame Options **********

    CharOwnedCountConfig_FrameOptionsTitle:SetText(L["CONFIG_HEADING_FRAME"]);

	CharOwnedCountConfig_FrameOptions_Duration.tooltip = L["OPTION_TOOLTIP_DURATION"];

	CharOwnedCountConfig_FrameOptions_LockText:SetText("Lock Counter Position");
    CharOwnedCountConfig_FrameOptions_Lock:SetChecked(true);

	CharOwnedCountConfig_FrameOptions_ResetPos:SetText(L["OPTION_RESETPOS"]);

    -- ********** Owned Count Options **********

    CharOwnedCountConfig_OwnedCountTitle:SetText(L["CONFIG_HEADING_COUNT"]);

    CharOwnedCountConfig_OwnedCount_ShowCount:SetText(L["OPTION_SHOWCOUNT"]);
    CharOwnedCountConfig_OwnedCount_ResetCount:SetText(L["OPTION_RESETCOUNT"]);

    CharOwnedCountConfig_OwnedCount_SeparateInstancesText:SetText(L["OPTION_SEPARATE_INSTANCE_COUNT"]);
    CharOwnedCountConfig_OwnedCount_SeparateInstances:SetChecked(true);

end

--========================================
-- Handle when the config panel is opened
--========================================
function CharOwnedConfig_OnShow()
    if (CONFIG_SHOWN) then return; end
    CharOwnedConfig_StoreCurrentSettings();
    CONFIG_SHOWN = true;
end

--========================================
-- Handle when the panel is refreshed
--========================================
function CharOwnedConfig_Refresh()
    CharOwnedConfig_OnShow();
	CharOwnedCountConfig_GeneralOptions_EnableSound:SetChecked(CHAR_OWNED['config']['enablesound']);
	CharOwnedCountConfig_GeneralOptions_LoadMessage:SetChecked(CHAR_OWNED['config']['show_load_message']);
	CharOwnedCountConfig_FrameOptions_Duration:SetValue(math.max(3, math.min(10, CHAR_OWNED['config']['duration'])));
end

--==================================================
-- Store current settings to restore if the user
-- presses cancel
--==================================================
function CharOwnedConfig_StoreCurrentSettings()
    OLD_ENABLE_SOUND = CHAR_OWNED['config']['enablesound'];
    OLD_LOAD_MESSAGE = CHAR_OWNED['config']['show_load_message'];
end

--========================================
-- Closing the config window
--========================================
function CharOwnedConfig_Close()
    CharOwned_LockFrame(true);
    CONFIG_SHOWN = false;
end

--==================================================
-- Handle clicking the Cancel button; restore
-- all settings to their previous values
--==================================================
function CharOwnedConfig_Cancel()
    CHAR_OWNED['config']['enablesound'] = OLD_ENABLE_SOUND;
    CHAR_OWNED['config']['show_load_message'] = OLD_LOAD_MESSAGE;
end

--========================================
-- Handler for checking/unchecking
-- checkbox(es)
--========================================
function CharOwnedConfig_CheckBox_OnClick(self, id)
	if (id == 1) then
		if (self:GetChecked()) then
		    CHAR_OWNED['config']['enablesound'] = true;
		else
		    CHAR_OWNED['config']['enablesound'] = false;
		end
    elseif (id == 2) then
		if (self:GetChecked()) then
		    CHAR_OWNED['config']['show_load_message'] = true;
		else
		    CHAR_OWNED['config']['show_load_message'] = false;
		end
    elseif (id == 10) then
        if (self:GetChecked()) then
            CharOwned_LockFrame();
        else
            CharOwned_UnlockFrame();
        end
    elseif (id == 20) then
        if (self:GetChecked()) then
            CHAR_OWNED['config']['separate_instance_counts'] = true;
        else
            CHAR_OWNED['config']['separate_instance_counts'] = false;
        end
	end
end

--========================================
-- Handler for changing the duration
-- slider value
--========================================
function CharOwnedConfig_Slider_OnValueChanged(self)
    local newValue = 0;
    if (not self._onsetting) then
        self._onsetting = true;
        self:SetValue(self:GetValue());
        newValue = math.floor(self:GetValue());
        self._onsetting = false;
    else return end
	CHAR_OWNED['config']['duration'] = self:GetValue();
	CharOwnedConfig_Slider_UpdateText();
end

--========================================
-- Slider text update for value changes
--========================================
function CharOwnedConfig_Slider_UpdateText()
    CharOwnedCountConfig_FrameOptions_DurationText1:SetText("Visibility Duration (seconds): " .. CharOwnedCountConfig_FrameOptions_Duration:GetValue());
end

--========================================
-- Handler for mousing over options
-- on the config window
--========================================
function CharOwnedConfig_Option_OnEnter(self)
	if (self.tooltip) then
        GameTooltip:SetOwner(self, "ANCHOR_NONE");
		GameTooltip:SetPoint("TOPLEFT", self:GetName(), "BOTTOMLEFT", -10, -4);
        GameTooltip:SetText(self.tooltip, 1, 1, 1);
        GameTooltip:Show();
	end
end

--========================================
-- Moving the mouse away from config
-- options
--========================================
function CharOwnedConfig_Option_OnLeave(self)
	GameTooltip:Hide();
end

--========================================
-- Config frame update
--========================================
function CharOwnedConfig_OnUpdate()

    if (not CharOwnedCountConfig_MuffinButton:IsEnabled()) then
        if ((GetTime() - CHAR_OWNED_LASTMUFFIN) > 5) then
            CharOwnedCountConfig_MuffinButton:Enable();
        end
    end

end

--========================================
-- Config-based lock toggling
--========================================
function CharOwnedConfig_ToggleLock()
    if (not CHAR_OWNED_UNLOCKED) then
        CharOwned_UnlockFrame(true);
	else
        CharOwned_LockFrame(true);
	end
end

--========================================
-- Clicking the Muffin Button
--========================================
function CharOwnedConfig_MuffinButton_Click(button)

    if (button) then
        button:Disable();
        CHAR_OWNED_LASTMUFFIN = GetTime();
    end

end
